@@ -1,32 +1,34 @@
 //! @file mp.c
 //! @author J. Marcel van der Veer
-//
+
 //! @section Copyright
-//
-// This file is part of Algol68G - an Algol 68 compiler-interpreter.
-// Copyright 2001-2023 J. Marcel van der Veer <algol68g@xs4all.nl>.
-//
+//!
+//! This file is part of Algol68G - an Algol 68 compiler-interpreter.
+//! Copyright 2001-2024 J. Marcel van der Veer [algol68g@xs4all.nl].
+
 //! @section License
-//
-// This program is free software; you can redistribute it and/or modify it 
-// under the terms of the GNU General Public License as published by the 
-// Free Software Foundation; either version 3 of the License, or 
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful, but 
-// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
-// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
-// more details. You should have received a copy of the GNU General Public 
-// License along with this program. If not, see <http://www.gnu.org/licenses/>.
+//!
+//! This program is free software; you can redistribute it and/or modify it 
+//! under the terms of the GNU General Public License as published by the 
+//! Free Software Foundation; either version 3 of the License, or 
+//! (at your option) any later version.
+//!
+//! This program is distributed in the hope that it will be useful, but 
+//! WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
+//! or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for 
+//! more details. You should have received a copy of the GNU General Public 
+//! License along with this program. If not, see [http://www.gnu.org/licenses/].
+
+//! @section Synopsis
+//!
+//! [LONG] LONG INT, REAL routines.
 
 // Multiprecision calculations are useful in these cases:
-//
 // Ill-conditioned linear systems
 // Summation of large series
 // Long-time or large-scale simulations
 // Small-scale phenomena
 // 'Experimental mathematics'
-//
 // The routines in this library follow algorithms as described in the
 // literature, notably
 // 
@@ -35,7 +37,6 @@
 // 
 //   D.M. Smith, "A Multiple-Precision Division Algorithm"
 //   Mathematics of Computation 66 (1996) 157-163
-//
 //   The GNU MPFR library documentation
 // 
 // Multiprecision libraries are (freely) available, but this one is particularly 
@@ -43,7 +44,7 @@
 // 
 //    LONG INT, LONG REAL, LONG COMPLEX, LONG BITS
 //    LONG LONG INT, LONG LONG REAL, LONG LONG COMPLEX, LONG LONG BITS
-//
+// 
 // Note that recent implementations of GCC make available 64-bit LONG INT and 
 // 128-bit LONG REAL. This suits many multiprecision needs already. 
 // On such platforms, below code is used for LONG LONG modes only.
@@ -56,7 +57,6 @@
 // user-definable precision. Precisions span about 30 decimal digits for
 // LONG modes up to (default) about 60 decimal digits for LONG LONG modes, a
 // range that is thought to be adequate for most multiprecision applications.
-//
 // Although the maximum length of a number is in principle unbound, this 
 // implementation is not designed for more than a few hundred decimal places. 
 // At higher precisions, expect a performance penalty with respect to
@@ -71,7 +71,6 @@
 // in a large base. Layout of a mp number "z" is:
 // 
 //    MP_T *z;
-//
 //    MP_STATUS (z)        Status word
 //    MP_EXPONENT (z)      Exponent with base MP_RADIX
 //    MP_DIGIT (z, 1 .. N) Digits 1 .. N
@@ -84,7 +83,7 @@
 // advantageous to keep the base as high as possible. Modern computers handle
 // doubles at similar or better speed as integers, therefore this library
 // opts for storing numbers as [] words were a word is real*8 (legacy) or 
-// int*8 (on f.i. ix86 processors that have real*10), trading space for speed.
+// int*8, trading space for speed.
 // 
 // Set a base such that "base^2" can be exactly represented by a word.
 // To facilitate transput, we require a base that is a power of 10.
@@ -104,7 +103,6 @@
 #include "a68g.h"
 #include "a68g-genie.h"
 #include "a68g-prelude.h"
-#include "a68g-double.h"
 #include "a68g-mp.h"
 
 // Internal mp constants.
@@ -120,19 +118,17 @@ void set_long_mp_digits (int n)
 
 int width_to_mp_digits (int n)
 {
-  return (int) ceil ((REAL_T) n / (REAL_T) LOG_MP_RADIX);
+  return 1 + n / LOG_MP_RADIX;
 }
 
 //! @brief Unformatted write of z to stdout; debugging routine.
 
-#if defined (A68_DEBUG)
 #if !defined (BUILD_WIN32)
 
 void raw_write_mp (char *str, MP_T * z, int digs)
 {
-  int i;
   fprintf (stdout, "\n(%d digits)%s", digs, str);
-  for (i = 1; i <= digs; i++) {
+  for (int i = 1; i <= digs; i++) {
 #if (A68_LEVEL >= 3)
     fprintf (stdout, " %09lld", (MP_INT_T) MP_DIGIT (z, i));
 #else
@@ -145,12 +141,11 @@ void raw_write_mp (char *str, MP_T * z, int digs)
   ASSERT (fflush (stdout) == 0);
 }
 
-#endif
 #endif
 
 //! @brief Whether z is a valid representation for its mode.
 
-BOOL_T check_mp_int (MP_T * z, MOID_T * m)
+BOOL_T check_mp_int (MP_T * z, const MOID_T * m)
 {
   if (m == M_LONG_INT || m == M_LONG_BITS) {
     return (BOOL_T) ((MP_EXPONENT (z) >= (MP_T) 0) && (MP_EXPONENT (z) < (MP_T) LONG_MP_DIGITS));
@@ -220,10 +215,9 @@ MP_T *plus_one_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
 
 BOOL_T same_mp (NODE_T * p, MP_T * x, MP_T * y, int digs)
 {
-  int k;
   (void) p;
   if ((MP_STATUS (x) == MP_STATUS (y)) && (MP_EXPONENT (x) == MP_EXPONENT (y))) {
-    for (k = digs; k >= 1; k--) {
+    for (int k = digs; k >= 1; k--) {
       if (MP_DIGIT (x, k) != MP_DIGIT (y, k)) {
         return A68_FALSE;
       }
@@ -248,7 +242,7 @@ MP_T *align_mp (MP_T * z, INT_T * expo, int digs)
     (*expo)--;
   }
 // Optimising below code does not make the library noticeably faster.
-  for (INT_T i = 1; i <= shift; i++) {
+  for (int i = 1; i <= shift; i++) {
     INT_T carry = 0;
     for (INT_T j = 1; j <= digs; j++) {
       MP_INT_T k = ((MP_INT_T) MP_DIGIT (z, j)) % 10;
@@ -350,8 +344,7 @@ MP_T *int_to_mp (NODE_T * p, MP_T * z, INT_T k, int digs)
     n++;
   }
   set_mp (z, 0, n, digs);
-  int j;
-  for (j = 1 + n; j >= 1; j--) {
+  for (int j = 1 + n; j >= 1; j--) {
     MP_DIGIT (z, j) = (MP_T) (k % MP_RADIX);
     k /= MP_RADIX;
   }
@@ -369,8 +362,7 @@ MP_T *unt_to_mp (NODE_T * p, MP_T * z, UNSIGNED_T k, int digs)
     n++;
   }
   set_mp (z, 0, n, digs);
-  int j;
-  for (j = 1 + n; j >= 1; j--) {
+  for (int j = 1 + n; j >= 1; j--) {
     MP_DIGIT (z, j) = (MP_T) (k % MP_RADIX);
     k /= MP_RADIX;
   }
@@ -392,8 +384,7 @@ INT_T mp_to_int (NODE_T * p, MP_T * z, int digs)
   if (negative) {
     MP_DIGIT (z, 1) = -MP_DIGIT (z, 1);
   }
-  int j;
-  for (j = 1 + expo; j >= 1; j--) {
+  for (int j = 1 + expo; j >= 1; j--) {
     if ((MP_INT_T) MP_DIGIT (z, j) > A68_MAX_INT / weight) {
       diagnostic (A68_RUNTIME_ERROR, p, ERROR_OUT_OF_BOUNDS, M_INT);
       exit_genie (p, A68_RUNTIME_ERROR);
@@ -433,8 +424,8 @@ MP_T *real_to_mp (NODE_T * p, MP_T * z, REAL_T x, int digs)
   }
 // Transport digs of x to the mantissa of z.
   INT_T sum = 0, weight = (MP_RADIX / 10);
-  int j = 1, k;
-  for (k = 0; a != 0.0 && j <= digs && k < REAL_DIGITS; k++) {
+  int j = 1;
+  for (int k = 0; a != 0.0 && j <= digs && k < A68_REAL_DIG; k++) {
     REAL_T u = a * 10;
     REAL_T v = floor (u);
     a = u - v;
@@ -463,15 +454,21 @@ REAL_T mp_to_real (NODE_T * p, MP_T * z, int digs)
 {
 // This routine looks a lot like "strtod".
   (void) p;
-  if (MP_EXPONENT (z) * (MP_T) LOG_MP_RADIX <= (MP_T) REAL_MIN_10_EXP) {
+  if (MP_EXPONENT (z) * (MP_T) LOG_MP_RADIX <= (MP_T) A68_REAL_MIN_EXP) {
     return 0;
   } else {
-    REAL_T sum = 0, weight = ten_up ((int) (MP_EXPONENT (z) * LOG_MP_RADIX));
-    int j;
-    for (j = 1; j <= digs && (j - 2) * LOG_MP_RADIX <= REAL_DIG; j++) {
-      sum += ABS (MP_DIGIT (z, j)) * weight;
+    REAL_T terms[1 + MP_MAX_DIGITS];
+    REAL_T weight = ten_up ((int) (MP_EXPONENT (z) * LOG_MP_RADIX));
+    unt lim = MIN (digs, MP_MAX_DIGITS);
+    for (int k = 1; k <= lim; k++) {
+      terms[k] = ABS (MP_DIGIT (z, k)) * weight;
       weight /= MP_RADIX;
     }
+// Sum terms from small to large.
+    REAL_T sum = 0;
+    for (int k = lim; k >= 1; k--) {
+      sum += terms[k];
+    }
     CHECK_REAL (p, sum);
     return MP_DIGIT (z, 1) >= 0 ? sum : -sum;
   }
@@ -483,8 +480,7 @@ static inline void norm_mp_light (MP_T * w, int k, int digs)
 {
 // Bring every digit back to [0 .. MP_RADIX>.
   MP_T *z = &MP_DIGIT (w, digs);
-  int j;
-  for (j = digs; j >= k; j--, z--) {
+  for (int j = digs; j >= k; j--, z--) {
     if (z[0] >= MP_RADIX) {
       z[0] -= (MP_T) MP_RADIX;
       z[-1] += 1;
@@ -500,9 +496,8 @@ static inline void norm_mp_light (MP_T * w, int k, int digs)
 static inline void norm_mp (MP_T * w, int k, int digs)
 {
 // Bring every digit back to [0 .. MP_RADIX>.
-  int j;
-  MP_T *z;
-  for (j = digs, z = &MP_DIGIT (w, digs); j >= k; j--, z--) {
+  MP_T *z = &MP_DIGIT (w, digs);
+  for (int j = digs; j >= k; j--, z--) {
     if (z[0] >= (MP_T) MP_RADIX) {
       MP_T carry = (MP_T) ((MP_INT_T) (z[0] / (MP_T) MP_RADIX));
       z[0] -= carry * (MP_T) MP_RADIX;
@@ -550,9 +545,8 @@ MP_T *trunc_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
     diagnostic (A68_RUNTIME_ERROR, p, ERROR_OUT_OF_BOUNDS, (IS (MOID (p), PROC_SYMBOL) ? SUB_MOID (p) : MOID (p)));
     exit_genie (p, A68_RUNTIME_ERROR);
   } else {
-    int k;
     (void) move_mp (z, x, digs);
-    for (k = (int) (MP_EXPONENT (x) + 2); k <= digs; k++) {
+    for (int k = (int) (MP_EXPONENT (x) + 2); k <= digs; k++) {
       MP_DIGIT (z, k) = (MP_T) 0;
     }
   }
@@ -570,9 +564,8 @@ MP_T *floor_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
     diagnostic (A68_RUNTIME_ERROR, p, ERROR_OUT_OF_BOUNDS, (IS (MOID (p), PROC_SYMBOL) ? SUB_MOID (p) : MOID (p)));
     exit_genie (p, A68_RUNTIME_ERROR);
   } else {
-    int k;
     (void) move_mp (z, x, digs);
-    for (k = (int) (MP_EXPONENT (x) + 2); k <= digs; k++) {
+    for (int k = (int) (MP_EXPONENT (x) + 2); k <= digs; k++) {
       MP_DIGIT (z, k) = (MP_T) 0;
     }
   }
@@ -582,16 +575,6 @@ MP_T *floor_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
   return z;
 }
 
-BOOL_T is_int_mp (NODE_T *p, MP_T *z, int digs)
-{
-  ADDR_T pop_sp = A68_SP;
-  MP_T *y = nil_mp (p, digs);
-  trunc_mp (p, y, z, digs);
-  BOOL_T tst = same_mp (p, y, z, digs);
-  A68_SP = pop_sp;
-  return tst;
-}
-
 //! @brief Shorten and round.
 
 MP_T *shorten_mp (NODE_T * p, MP_T * z, int digs, MP_T * x, int digs_x)
@@ -636,8 +619,7 @@ MP_T *lengthen_mp (NODE_T * p, MP_T * z, int digs_z, MP_T * x, int digs_x)
       MP_EXPONENT (z) = MP_EXPONENT (x);
       MP_STATUS (z) = MP_STATUS (x);
     }
-    int j;
-    for (j = 1 + digs_x; j <= digs_z; j++) {
+    for (int j = 1 + digs_x; j <= digs_z; j++) {
       MP_DIGIT (z, j) = (MP_T) 0;
     }
   }
@@ -675,24 +657,23 @@ MP_T *add_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
     MP_T *w = nil_mp (p, digs_h);
     if (MP_EXPONENT (x) == MP_EXPONENT (y)) {
       MP_EXPONENT (w) = (MP_T) 1 + MP_EXPONENT (x);
-      int j;
-      for (j = 1; j <= digs; j++) {
+      for (int j = 1; j <= digs; j++) {
         MP_DIGIT (w, j + 1) = MP_DIGIT (x, j) + MP_DIGIT (y, j);
       }
       MP_DIGIT (w, digs_h) = (MP_T) 0;
     } else if (MP_EXPONENT (x) > MP_EXPONENT (y)) {
-      int j, shl_y = (int) MP_EXPONENT (x) - (int) MP_EXPONENT (y);
+      int shl_y = (int) MP_EXPONENT (x) - (int) MP_EXPONENT (y);
       MP_EXPONENT (w) = (MP_T) 1 + MP_EXPONENT (x);
-      for (j = 1; j < digs_h; j++) {
+      for (int j = 1; j < digs_h; j++) {
         int i_y = j - shl_y;
         MP_T x_j = (j > digs ? 0 : MP_DIGIT (x, j));
         MP_T y_j = (i_y <= 0 || i_y > digs ? 0 : MP_DIGIT (y, i_y));
         MP_DIGIT (w, j + 1) = x_j + y_j;
       }
     } else {
-      int j, shl_x = (int) MP_EXPONENT (y) - (int) MP_EXPONENT (x);
+      int shl_x = (int) MP_EXPONENT (y) - (int) MP_EXPONENT (x);
       MP_EXPONENT (w) = (MP_T) 1 + MP_EXPONENT (y);
-      for (j = 1; j < digs_h; j++) {
+      for (int j = 1; j < digs_h; j++) {
         int i_x = j - shl_x;
         MP_T x_j = (i_x <= 0 || i_x > digs ? 0 : MP_DIGIT (x, i_x));
         MP_T y_j = (j > digs ? 0 : MP_DIGIT (y, j));
@@ -741,18 +722,18 @@ MP_T *sub_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
   } else {
 // Subtract.
     BOOL_T negative = A68_FALSE;
-    int j, fnz, digs_h = 2 + digs;
+    int fnz, digs_h = 2 + digs;
     MP_T *w = nil_mp (p, digs_h);
     if (MP_EXPONENT (x) == MP_EXPONENT (y)) {
       MP_EXPONENT (w) = (MP_T) 1 + MP_EXPONENT (x);
-      for (j = 1; j <= digs; j++) {
+      for (int j = 1; j <= digs; j++) {
         MP_DIGIT (w, j + 1) = MP_DIGIT (x, j) - MP_DIGIT (y, j);
       }
       MP_DIGIT (w, digs_h) = (MP_T) 0;
     } else if (MP_EXPONENT (x) > MP_EXPONENT (y)) {
       int shl_y = (int) MP_EXPONENT (x) - (int) MP_EXPONENT (y);
       MP_EXPONENT (w) = (MP_T) 1 + MP_EXPONENT (x);
-      for (j = 1; j < digs_h; j++) {
+      for (int j = 1; j < digs_h; j++) {
         int i_y = j - shl_y;
         MP_T x_j = (j > digs ? 0 : MP_DIGIT (x, j));
         MP_T y_j = (i_y <= 0 || i_y > digs ? 0 : MP_DIGIT (y, i_y));
@@ -761,7 +742,7 @@ MP_T *sub_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
     } else {
       int shl_x = (int) MP_EXPONENT (y) - (int) MP_EXPONENT (x);
       MP_EXPONENT (w) = (MP_T) 1 + MP_EXPONENT (y);
-      for (j = 1; j < digs_h; j++) {
+      for (int j = 1; j < digs_h; j++) {
         int i_x = j - shl_x;
         MP_T x_j = (i_x <= 0 || i_x > digs ? 0 : MP_DIGIT (x, i_x));
         MP_T y_j = (j > digs ? 0 : MP_DIGIT (y, j));
@@ -771,14 +752,14 @@ MP_T *sub_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
 // Correct if we subtract large from small.
     if (MP_DIGIT (w, 2) <= 0) {
       fnz = -1;
-      for (j = 2; j <= digs_h && fnz < 0; j++) {
+      for (int j = 2; j <= digs_h && fnz < 0; j++) {
         if (MP_DIGIT (w, j) != 0) {
           fnz = j;
         }
       }
       negative = (BOOL_T) (MP_DIGIT (w, fnz) < 0);
       if (negative) {
-        for (j = fnz; j <= digs_h; j++) {
+        for (int j = fnz; j <= digs_h; j++) {
           MP_DIGIT (w, j) = -MP_DIGIT (w, j);
         }
       }
@@ -786,14 +767,14 @@ MP_T *sub_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
 // Normalise.
     norm_mp_light (w, 2, digs_h);
     fnz = -1;
-    for (j = 1; j <= digs_h && fnz < 0; j++) {
+    for (int j = 1; j <= digs_h && fnz < 0; j++) {
       if (MP_DIGIT (w, j) != 0) {
         fnz = j;
       }
     }
     if (fnz > 1) {
       int j2 = fnz - 1;
-      for (j = 1; j <= digs_h - j2; j++) {
+      for (int j = 1; j <= digs_h - j2; j++) {
         MP_DIGIT (w, j) = MP_DIGIT (w, j + j2);
         MP_DIGIT (w, j + j2) = (MP_T) 0;
       }
@@ -825,14 +806,14 @@ MP_T *mul_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
   }
 // Grammar school algorithm with intermittent normalisation.
   ADDR_T pop_sp = A68_SP;
-  int i, digs_h = 2 + digs;
+  int digs_h = 2 + digs;
   MP_T x_1 = MP_DIGIT (x, 1), y_1 = MP_DIGIT (y, 1);
   MP_DIGIT (x, 1) = ABS (x_1);
   MP_DIGIT (y, 1) = ABS (y_1);
   MP_STATUS (z) = (MP_T) INIT_MASK;
   MP_T *w = lit_mp (p, 0, MP_EXPONENT (x) + MP_EXPONENT (y) + 1, digs_h);
   int oflow = (int) FLOOR_MP ((MP_REAL_T) MAX_REPR_INT / (2 * MP_REAL_RADIX * MP_REAL_RADIX)) - 1;
-  for (i = digs; i >= 1; i--) {
+  for (int i = digs; i >= 1; i--) {
     MP_T yi = MP_DIGIT (y, i);
     if (yi != 0) {
       int k = digs_h - i;
@@ -870,10 +851,8 @@ MP_T *div_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
 // This is O(N^2) but runs faster than straightforward methods by skipping
 // most of the intermediate normalisation and recovering from wrong
 // guesses without separate correction steps.
-//
 // Depending on application, div_mp cost is circa 3 times that of mul_mp.
 // Therefore Newton-Raphson division makes no sense here.
-//
   if (IS_ZERO_MP (y)) {
     errno = ERANGE;
     return NaN_MP;
@@ -884,7 +863,6 @@ MP_T *div_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
 #else
   int oflow = (int) FLOOR_MP ((MP_REAL_T) MAX_REPR_INT / (2 * MP_REAL_RADIX * MP_REAL_RADIX)) - 1;
 #endif
-//
   MP_T x_1 = MP_DIGIT (x, 1), y_1 = MP_DIGIT (y, 1);
   MP_DIGIT (x, 1) = ABS (x_1);
   MP_DIGIT (y, 1) = ABS (y_1);
@@ -911,8 +889,7 @@ MP_T *div_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
 // Estimate the denominator. For small MP_RADIX add: MP_DIGIT (y, 4) / MP_REAL_RADIX.
   MP_REAL_T den = (MP_DIGIT (y, 1) * MP_REAL_RADIX + MP_DIGIT (y, 2)) * MP_REAL_RADIX + MP_DIGIT (y, 3);
   MP_T *t = &MP_DIGIT (w, 2);
-  int k, len, first;
-  for (k = 1, len = digs + 2, first = 3; k <= digs + 2; k++, len++, first++, t++) {
+  for (int k = 1, len = digs + 2, first = 3; k <= digs + 2; k++, len++, first++, t++) {
 // Estimate quotient digit.
     MP_REAL_T q, nom = ((t[-1] * MP_REAL_RADIX + t[0]) * MP_REAL_RADIX + t[1]) * MP_REAL_RADIX + (wdigs >= (first + 2) ? t[2] : 0);
     if (nom == 0) {
@@ -925,8 +902,7 @@ MP_T *div_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
         lim = first + nzdigs - 1;
       }
       MP_T *u = t, *v = &MP_DIGIT (y, 1);
-      int j;
-      for (j = first; j <= lim; j++) {
+      for (int j = first; j <= lim; j++) {
         (u++)[0] -= q * (v++)[0];
       }
     }
@@ -956,16 +932,13 @@ MP_T *over_mp (NODE_T * p, MP_T * z, MP_T * x, MP_T * y, int digs)
     errno = ERANGE;
     return NaN_MP;
   }
-  int digs_g = FUN_DIGITS (digs);
   ADDR_T pop_sp = A68_SP;
-  MP_T *x_g = len_mp (p, x, digs, digs_g);
-  MP_T *y_g = len_mp (p, y, digs, digs_g);
-  MP_T *z_g = nil_mp (p, digs_g);
+  int digs_g = FUN_DIGITS (digs);
+  MP_T *x_g = len_mp (p, x, digs, digs_g), *y_g = len_mp (p, y, digs, digs_g), *z_g = nil_mp (p, digs_g);
   (void) div_mp (p, z_g, x_g, y_g, digs_g);
   trunc_mp (p, z_g, z_g, digs_g);
   (void) shorten_mp (p, z, digs, z_g, digs_g);
   MP_STATUS (z) = (MP_T) INIT_MASK;
-// Restore and exit.
   A68_SP = pop_sp;
   return z;
 }
@@ -1030,16 +1003,15 @@ MP_T *mul_mp_digit (NODE_T * p, MP_T * z, MP_T * x, MP_T y, int digs)
 
 MP_T *half_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
 {
-  MP_T *w, z_1, x_1 = MP_DIGIT (x, 1), *u, *v;
-  int j, digs_h = 2 + digs;
   ADDR_T pop_sp = A68_SP;
+  int digs_h = 2 + digs;
+  MP_T x_1 = MP_DIGIT (x, 1);
   MP_DIGIT (x, 1) = ABS (x_1);
   MP_STATUS (z) = (MP_T) INIT_MASK;
 // Calculate x * 0.5.
-  w = lit_mp (p, 0, MP_EXPONENT (x), digs_h);
-  j = digs;
-  u = &MP_DIGIT (w, 1 + digs);
-  v = &MP_DIGIT (x, digs);
+  MP_T *w = lit_mp (p, 0, MP_EXPONENT (x), digs_h);
+  MP_T *u = &MP_DIGIT (w, 1 + digs), *v = &MP_DIGIT (x, digs);
+  int j = digs;
   while (j-- >= 1) {
     (u--)[0] += (MP_RADIX / 2) * (v--)[0];
   }
@@ -1047,7 +1019,7 @@ MP_T *half_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
   round_internal_mp (z, w, digs);
 // Restore and exit.
   A68_SP = pop_sp;
-  z_1 = MP_DIGIT (z, 1);
+  MP_T z_1 = MP_DIGIT (z, 1);
   MP_DIGIT (x, 1) = x_1;
   MP_DIGIT (z, 1) = (x_1 >= 0 ? z_1 : -z_1);
   check_mp_exp (p, z);
@@ -1058,16 +1030,15 @@ MP_T *half_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
 
 MP_T *tenth_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
 {
-  MP_T *w, z_1, x_1 = MP_DIGIT (x, 1), *u, *v;
-  int j, digs_h = 2 + digs;
   ADDR_T pop_sp = A68_SP;
+  int digs_h = 2 + digs;
+  MP_T x_1 = MP_DIGIT (x, 1);
   MP_DIGIT (x, 1) = ABS (x_1);
   MP_STATUS (z) = (MP_T) INIT_MASK;
 // Calculate x * 0.1.
-  w = lit_mp (p, 0, MP_EXPONENT (x), digs_h);
-  j = digs;
-  u = &MP_DIGIT (w, 1 + digs);
-  v = &MP_DIGIT (x, digs);
+  MP_T *w = lit_mp (p, 0, MP_EXPONENT (x), digs_h);
+  MP_T *u = &MP_DIGIT (w, 1 + digs), *v = &MP_DIGIT (x, digs);
+  int j = digs;
   while (j-- >= 1) {
     (u--)[0] += (MP_RADIX / 10) * (v--)[0];
   }
@@ -1075,7 +1046,7 @@ MP_T *tenth_mp (NODE_T * p, MP_T * z, MP_T * x, int digs)
   round_internal_mp (z, w, digs);
 // Restore and exit.
   A68_SP = pop_sp;
-  z_1 = MP_DIGIT (z, 1);
+  MP_T z_1 = MP_DIGIT (z, 1);
   MP_DIGIT (x, 1) = x_1;
   MP_DIGIT (z, 1) = (x_1 >= 0 ? z_1 : -z_1);
   check_mp_exp (p, z);
@@ -1102,19 +1073,18 @@ MP_T *div_mp_digit (NODE_T * p, MP_T * z, MP_T * x, MP_T y, int digs)
   MP_DIGIT (x, 1) = ABS (x_1);
   MP_STATUS (z) = (MP_T) INIT_MASK;
   y = ABS (y_1);
-//
   if (y == 2) {
     (void) half_mp (p, z, x, digs);
   } else if (y == 10) {
     (void) tenth_mp (p, z, x, digs);
   } else {
-    int k, first, wdigs = 4 + digs;
+    int wdigs = 4 + digs;
     MP_T *w = lit_mp (p, 0, MP_EXPONENT (x), wdigs);
     (void) move_mp_part (&MP_DIGIT (w, 2), &MP_DIGIT (x, 1), digs);
 // Estimate the denominator.
     MP_REAL_T den = (MP_REAL_T) y * MP_REAL_RADIX * MP_REAL_RADIX;
     MP_T *t = &MP_DIGIT (w, 2);
-    for (k = 1, first = 3; k <= digs + 2; k++, first++, t++) {
+    for (int k = 1, first = 3; k <= digs + 2; k++, first++, t++) {
 // Estimate quotient digit and correct.
       MP_REAL_T nom = ((t[-1] * MP_REAL_RADIX + t[0]) * MP_REAL_RADIX + t[1]) * MP_REAL_RADIX + (wdigs >= (first + 2) ? t[2] : 0);
       MP_REAL_T q = (MP_T) (MP_INT_T) (nom / den);
@@ -1146,8 +1116,7 @@ MP_T *over_mp_digit (NODE_T * p, MP_T * z, MP_T * x, MP_T y, int digs)
   }
   int digs_g = FUN_DIGITS (digs);
   ADDR_T pop_sp = A68_SP;
-  MP_T *x_g = len_mp (p, x, digs, digs_g);
-  MP_T *z_g = nil_mp (p, digs_g);
+  MP_T *x_g = len_mp (p, x, digs, digs_g), *z_g = nil_mp (p, digs_g);
   (void) div_mp_digit (p, z_g, x_g, y, digs_g);
   trunc_mp (p, z_g, z_g, digs_g);
   (void) shorten_mp (p, z, digs, z_g, digs_g);
@@ -1186,15 +1155,13 @@ MP_T *pow_mp_int (NODE_T * p, MP_T * z, MP_T * x, INT_T n, int digs)
 {
   ADDR_T pop_sp = A68_SP;
   int bit, digs_g = FUN_DIGITS (digs);
-  BOOL_T negative;
-  MP_T *x_g = len_mp (p, x, digs, digs_g);
-  MP_T *z_g = lit_mp (p, 1, 0, digs_g);
-  negative = (BOOL_T) (n < 0);
+  MP_T *x_g = len_mp (p, x, digs, digs_g), *z_g = lit_mp (p, 1, 0, digs_g);
+  BOOL_T negative = (BOOL_T) (n < 0);
   if (negative) {
     n = -n;
   }
   bit = 1;
-  while ((unt) bit <= (unt) n) {
+  while ((int) bit <= (int) n) {
     if (n & bit) {
       (void) mul_mp (p, z_g, z_g, x_g, digs_g);
     }
