@@ -1765,7 +1765,13 @@ void genie_write_standard_format (NODE_T * p, MOID_T * mode, BYTE_T * item, A68_
     }
   } else if (IS_UNION (mode)) {
     A68_UNION *z = (A68_UNION *) item;
-    genie_write_standard_format (p, (MOID_T *) (VALUE (z)), &item[A68_UNION_SIZE], ref_file, formats);
+    MOID_T *um = (MOID_T *) (VALUE (z));
+    BYTE_T *ui = &item[A68_UNION_SIZE];
+    if (um == NO_MOID) {
+      diagnostic (A68_RUNTIME_ERROR, p, ERROR_EMPTY_VALUE, mode);
+      exit_genie (p, A68_RUNTIME_ERROR);
+    }
+    genie_write_standard_format (p, um, ui, ref_file, formats);
   } else if (IS_STRUCT (mode)) {
     for (PACK_T *q = PACK (mode); q != NO_PACK; FORWARD (q)) {
       BYTE_T *elem = &item[OFFSET (q)];
